/*
 * Copyright 2002-2007 Stuart Gunter.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.stuartgunter.rep.annotations;

import java.lang.annotation.*;

/**
 * @author Stuart Gunter
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({ ElementType.METHOD, ElementType.TYPE })
@Documented
public @interface Robot {

    /**
     * Specifies the user agent that these directives apply to. If not specified, the directives will apply to all
     * user agents.
     */
    String userAgent() default "";

    /**
     * Specifies the directives that are to be applied to the specified user agent.
     */
    RobotsDirective[] directives();

    /**
     * The period of time after which this content is unavailable. This requires the use of the
     * {@see RobotsDirective.UNAVAILABLE_AFTER} directive.
     *
     * The period is to be specified in the Joda Time period format.
     */
    String unavailableAfter() default "";
}
