package org.stuartgunter.rep.formatters;

import com.google.common.annotations.VisibleForTesting;
import org.joda.time.DateTime;
import org.joda.time.Period;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.joda.time.format.PeriodFormat;
import org.joda.time.format.PeriodFormatter;
import org.springframework.stereotype.Component;
import org.stuartgunter.rep.annotations.RobotsDirective;

import static com.google.common.base.Preconditions.checkArgument;

/**
 * @author Stuart Gunter
 */
@Component
public class RobotDirectivesFormatter {

    public static final String RFC_850_DATE_FORMAT = "dd MMM yy HH:mm:ss zzz";

    private static final DateTimeFormatter DATE_TIME_FORMATTER = DateTimeFormat.forPattern(RFC_850_DATE_FORMAT);
    private static final PeriodFormatter PERIOD_FORMATTER = PeriodFormat.getDefault();

    String format(RobotInfo robot) {
        final StringBuilder value = new StringBuilder();
        int remaining = robot.getDirectives().size();
        for (RobotsDirective directive : robot.getDirectives()) {
            value.append(directive.getDirective());

            if (directive == RobotsDirective.UNAVAILABLE_AFTER) {
                checkArgument(robot.getUnavailableAfter() != null, "Missing unavailableAfter date");

                final Period period = periodFormatter().parsePeriod(robot.getUnavailableAfter());
                final DateTime unavailableAfter = DateTime.now().plus(period);

                value.append(": ").append(dateTimeFormatter().print(unavailableAfter));
            }

            if (--remaining > 0) {
                value.append(", ");
            }
        }

        return value.toString();
    }

    @VisibleForTesting
    protected DateTimeFormatter dateTimeFormatter() {
        return DATE_TIME_FORMATTER;
    }

    @VisibleForTesting
    protected PeriodFormatter periodFormatter() {
        return PERIOD_FORMATTER;
    }
}
