package org.stuartgunter.rep.formatters;

import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;
import org.stuartgunter.rep.annotations.RobotsDirective;

import java.util.Collection;
import java.util.Set;

/**
 * This class includes all the information required for the Robots Exclusion Protocol.
 *
 * @author Stuart Gunter
 */
public class RobotInfo {

    private final String userAgent;
    private final Set<RobotsDirective> directives = Sets.newTreeSet();
    private String unavailableAfter;

    public RobotInfo(String userAgent) {
        this.userAgent = userAgent;
    }

    public String getUserAgent() {
        return userAgent;
    }

    public Set<RobotsDirective> getDirectives() {
        return ImmutableSet.copyOf(directives);
    }

    public void addDirectives(Collection<RobotsDirective> directives) {
        this.directives.addAll(directives);
    }

    public String getUnavailableAfter() {
        return unavailableAfter;
    }

    public void setUnavailableAfter(String unavailableAfter) {
        this.unavailableAfter = unavailableAfter;
    }
}
