package org.stuartgunter.rep.interceptor;

import org.stuartgunter.rep.annotations.Robot;
import org.stuartgunter.rep.annotations.Robots;
import org.stuartgunter.rep.formatters.RobotInfo;

import java.lang.reflect.Method;
import java.util.List;
import java.util.Stack;

/**
 * A strategy for resolving the correct list of {@link RobotInfo} items from a {@link Method}.
 *
 * @author Stuart Gunter
 */
public abstract class RobotCombinatorStrategy {

    /**
     * Finds relevant {@link org.stuartgunter.rep.annotations.Robot} and {@link org.stuartgunter.rep.annotations.Robots}
     * definitions and combines them into a list of {@link RobotInfo} items.
     * @param target The {@link Method} that serves as the starting point for finding annotations
     * @return A combined list of {@link RobotInfo} items
     */
    public abstract List<RobotInfo> combine(Method target);

    protected Stack<Robot> findAllAnnotations(Method target) {
        Stack<Robot> annotations = new Stack<Robot>();

        Robot methodRobot = target.getAnnotation(Robot.class);
        if (methodRobot != null) {
            annotations.push(methodRobot);
        }

        Robots methodRobots = target.getAnnotation(Robots.class);
        if (methodRobots != null) {
            for (Robot robot : methodRobots.value()) {
                annotations.push(robot);
            }
        }

        Class<?> currentClass = target.getDeclaringClass();
        while (currentClass != null) {
            Robot classRobot = currentClass.getAnnotation(Robot.class);
            if (classRobot != null) {
                annotations.push(classRobot);
            }

            Robots classRobots = currentClass.getAnnotation(Robots.class);
            if (classRobots != null) {
                for (Robot robot : classRobots.value()) {
                    annotations.push(robot);
                }
            }

            currentClass = currentClass.getSuperclass();
        }

        return annotations;
    }
}
