package org.stuartgunter.rep.interceptor;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.springframework.util.StringUtils;
import org.stuartgunter.rep.annotations.Robot;
import org.stuartgunter.rep.formatters.RobotInfo;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Stack;

/**
 * This strategy merges all {@link org.stuartgunter.rep.annotations.Robot} and
 * {@link org.stuartgunter.rep.annotations.Robots} annotations to produce a list of
 * {@link RobotInfo} items. Where definitions conflict, the closest definition takes precedence.
 *
 * @author Stuart Gunter
 */
public class RobotMergeCombinator extends RobotCombinatorStrategy {

    @Override
    public List<RobotInfo> combine(Method target) {
        final Stack<Robot> robots = findAllAnnotations(target);
        return mergeRobots(robots);
    }

    private List<RobotInfo> mergeRobots(Stack<Robot> robots) {
        Map<String, RobotInfo> robotInfo = Maps.newHashMap();

        Robot robot = robots.pop();
        while (robot != null) {
            RobotInfo info = robotInfo.get(robot.userAgent());

            if (info == null) {
                info = new RobotInfo(robot.userAgent());
                robotInfo.put(robot.userAgent(), info);
            }

            if (StringUtils.hasText(robot.unavailableAfter())) {
                info.setUnavailableAfter(robot.unavailableAfter());
            }
            
            info.addDirectives(Arrays.asList(robot.directives()));

            robot = robots.empty() ? null : robots.pop();
        }

        return Lists.newArrayList(robotInfo.values());
    }
}
