package org.stuartgunter.rep.interceptor;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.springframework.util.StringUtils;
import org.stuartgunter.rep.annotations.Robot;
import org.stuartgunter.rep.formatters.RobotInfo;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Stack;

/**
 * This strategy allows {@link org.stuartgunter.rep.annotations.Robot} and
 * {@link org.stuartgunter.rep.annotations.Robots} annotations to be overridden when defined at different
 * levels. More specific definitions will entirely override previous definitions for the same user agent.
 *
 * @author Stuart Gunter
 */
public class RobotOverrideCombinator extends RobotCombinatorStrategy {

    @Override
    public List<RobotInfo> combine(Method target) {
        final Stack<Robot> robots = findAllAnnotations(target);
        return overrideRobots(robots);
    }

    private List<RobotInfo> overrideRobots(Stack<Robot> robots) {
        Map<String, RobotInfo> robotInfo = Maps.newHashMap();

        Robot robot = robots.pop();
        while (robot != null) {
            RobotInfo info = new RobotInfo(robot.userAgent());
            info.addDirectives(Arrays.asList(robot.directives()));
            if (StringUtils.hasText(robot.unavailableAfter())) {
                info.setUnavailableAfter(robot.unavailableAfter());
            }

            robotInfo.put(robot.userAgent(), info);

            robot = robots.empty() ? null : robots.pop();
        }

        return Lists.newArrayList(robotInfo.values());
    }
}
